#include "common.h"
#include "player.h"
#include <windows.h>
#include "args.h"
#include <mmsystem.h>
#include <math.h>
#include "stream.h"

DWORD PlayerThreadID = 0;
HANDLE PlayThreadHan = 0;                                          

// Copy of Args->OutMode, set in constructor
esOutputMode OutMode;
// Stuff for the wavfile renderer
HMMIO    hmmioOut;
MMIOINFO mmioinfoOut;
MMCKINFO ckOutRIFF;
MMCKINFO ckOut;

long BufferCount;
bool FirstBuf;

WAVEFORMATEX WAVFx, outFormatex;
HWAVEOUT WaveDevice;

bool MPAudioEvents[4];

bool Audio_Initialized = false;

void ProcessAudioMessages()
{
    MSG msg;
    HWAVEOUT Wave;
    WAVEHDR *Wh; 
    HGLOBAL WHMemHnd,
                    DataMemHnd;
    while (PeekMessage(&msg, (HWND) -1, 0, 0, PM_NOREMOVE)) 
    {
        GetMessage(&msg,  (HWND) NULL,  0, 0);
        if (msg.message == MM_WOM_DONE)
        {
            Wave = (HWAVEOUT) msg.wParam;
            Wh = (WAVEHDR *) msg.lParam;
            WHMemHnd=GlobalHandle(Wh);
        
            DataMemHnd=GlobalHandle(Wh->lpData);
            waveOutUnprepareHeader((HWAVEOUT)Wave, Wh, sizeof(WAVEHDR));
            GlobalUnlock(DataMemHnd);
            GlobalUnlock(WHMemHnd);
            GlobalFree(DataMemHnd);
            GlobalFree(WHMemHnd);
            BufferCount--;          
        } 
        else 
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }    
}


bool Init_MPAudio(){
    OutMode = Args->OutputMode;
	FirstBuf=true;
    if(OutMode==omMMSystem) {

        if (!waveOutGetNumDevs()) {
            throw 1;
        } 
        BufferCount = 0;
        outFormatex.wFormatTag        = WAVE_FORMAT_PCM;
        outFormatex.wBitsPerSample    = Args->AudioBits;
        outFormatex.nChannels         = Args->Channels;

        outFormatex.nSamplesPerSec    = Args->SampleRate;
        
        outFormatex.nAvgBytesPerSec   = outFormatex.nSamplesPerSec*Args->Channels*(Args->AudioBits/8);
        outFormatex.nBlockAlign       = Args->Channels*(Args->AudioBits/8);
              
        for(int i=0;i<4;i++) MPAudioEvents[i]=false;
        MMRESULT res=waveOutOpen(&WaveDevice,(UINT)Args->AudioDevice,&outFormatex, PlayerThreadID, 0, CALLBACK_THREAD);
        if(res!=MMSYSERR_NOERROR)
        {
            throw 1;
        }
        res=waveOutReset(WaveDevice);
        if(res!=MMSYSERR_NOERROR)
        {
            throw 1;
        }
        ProcessAudioMessages();
        Sleep(77);
        Audio_Initialized = true;
        return true;
    }
    if(OutMode==omWaveFile) 
    {
        if(!(hmmioOut=mmioOpen(Args->OutName,NULL,MMIO_ALLOCBUF|MMIO_WRITE|MMIO_CREATE))) {
            throw 1;
        }
        ckOutRIFF.fccType=mmioFOURCC('W', 'A', 'V', 'E');
        ckOutRIFF.cksize=0;
        if(mmioCreateChunk(hmmioOut,&ckOutRIFF,MMIO_CREATERIFF)!=MMSYSERR_NOERROR) {
            throw 1;
        }
        WAVFx.wFormatTag        = WAVE_FORMAT_PCM;
        WAVFx.wBitsPerSample    = Args->AudioBits;
        WAVFx.nChannels         = Args->Channels;
        WAVFx.nSamplesPerSec    = Args->SampleRate;
        WAVFx.nAvgBytesPerSec   = WAVFx.nSamplesPerSec*WAVFx.nChannels*WAVFx.wBitsPerSample/8;
        WAVFx.nBlockAlign       = WAVFx.nChannels*WAVFx.wBitsPerSample/8;
        WAVFx.cbSize          = 0;
        ckOut.ckid=mmioFOURCC('f', 'm', 't', ' ');
        ckOut.cksize=sizeof(WAVFx);
        if(mmioCreateChunk(hmmioOut,&ckOut,0)!=MMSYSERR_NOERROR) {
            throw 1;
        }
        if(mmioWrite(hmmioOut,(HPSTR)&WAVFx,sizeof(WAVFx))!=sizeof(WAVFx)){
            throw 1;
        }
        if(mmioAscend(hmmioOut,&ckOut,0)!=MMSYSERR_NOERROR) {
            throw 1;
        }
        ckOut.ckid=mmioFOURCC('d', 'a', 't', 'a');
        ckOut.cksize=0;
        if(mmioCreateChunk(hmmioOut,&ckOut,0)!=MMSYSERR_NOERROR){
            throw 1;
        }
        Audio_Initialized = true;
        return true;
    }
    return false;
}

long PlaySamples(unsigned char *buf, long len){
	if(FirstBuf)
	{
		FirstBuf=false;
		if(len>3500)
		{
			buf+=3500;
			len-=3500;
		}
	}

    if(OutMode==omMMSystem)
    {
        LPWAVEHDR WaveHeader; 
        HGLOBAL WHMemHnd,DataMemHnd;
        if ((MPAudioEvents[0]) || (MPAudioEvents[2])) return -1; 
        ProcessAudioMessages();
        while (BufferCount >= Args->Buffers) 
        {
            Sleep(77);
            if ((MPAudioEvents[0]) || (MPAudioEvents[2])) return -1;
            ProcessAudioMessages();
        }
        WHMemHnd = GlobalAlloc(GMEM_MOVEABLE,sizeof(WAVEHDR));
        DataMemHnd=GlobalAlloc(GMEM_MOVEABLE,len);
        WaveHeader=(LPWAVEHDR)GlobalLock(WHMemHnd);
        WaveHeader->lpData=(LPSTR)GlobalLock(DataMemHnd);
        try 
        {
          CopyMemory(WaveHeader->lpData,buf,len);
        }
        catch (...)
        {
            GlobalFree(DataMemHnd);
            GlobalFree(WHMemHnd);  
            throw 1;   
        }
        WaveHeader->dwBufferLength=len;
        WaveHeader->dwFlags=0;
        MMRESULT Res;
        try 
        {
          Res=waveOutPrepareHeader(WaveDevice,WaveHeader,sizeof(WAVEHDR));
        }
        catch (...)
        {
            GlobalFree(DataMemHnd);
            GlobalFree(WHMemHnd);  
            throw 1;   
        }
        if(Res) 
        {
            GlobalFree(DataMemHnd);
            GlobalFree(WHMemHnd);  
            throw 1;            
        }
        Res = waveOutWrite(WaveDevice,WaveHeader,sizeof(WAVEHDR));
        if(Res) {
            GlobalFree(DataMemHnd);
            GlobalFree(WHMemHnd);  
            throw 1;            
        }
        BufferCount++;
        return(len);
    }
    if(OutMode == omWaveFile) 
    {
        unsigned long i;
        if ((MPAudioEvents[0]) || (MPAudioEvents[2])) return -1; 
        if(MPAudioEvents[1]){
            MPAudioEvents[1]=false;
        }         
        i = mmioWrite(hmmioOut, (const char *) buf, len);
        Sleep(20);
        return i;
    }
    return -1;
} 

void Pause_Audio(){
    if(OutMode==omMMSystem)
    {
        MPAudioEvents[0]=true;
        waveOutPause(WaveDevice);
    } 
}

void Resume_Audio(){
    if(OutMode==omMMSystem) {
        MPAudioEvents[0]=false; 
        waveOutRestart(WaveDevice);
    } 
}

void FinishBuffer()
{
    if (OutMode==omMMSystem) 
    {
        ProcessAudioMessages();
        while(BufferCount) 
        {
            Sleep(77);
            ProcessAudioMessages();
        }
    }
}

bool Done_MPAudio(){
    Audio_Initialized = false;
    if(OutMode==omMMSystem) 
    {
        try
        {
            while (waveOutClose (WaveDevice) == WAVERR_STILLPLAYING) 
            {
                ProcessAudioMessages();
                Sleep(77);
            }
            ProcessAudioMessages();
        }
        catch (...)
		{
		}
        WaveDevice=NULL;
        BufferCount=0;
    }
    if(OutMode==omWaveFile) 
    {
        if(mmioFlush(hmmioOut,MMIO_EMPTYBUF)) 
        {
            return true;
        }
        if(mmioAscend(hmmioOut,&ckOut,0)!=MMSYSERR_NOERROR)
        {
            return true;
        }
        if(mmioAscend(hmmioOut,&ckOutRIFF,0)!=MMSYSERR_NOERROR)
        {
            return true;
        }
        if(mmioClose(hmmioOut,0)!=MMSYSERR_NOERROR) 
        {
            return true;
        }
        return true;
    }
    return true;
}

