#include "common.h"
#include "args.h"
#include "player.h"
#include "helper.h"
#include "stream.h"
#include "audio.h"
#include <stdio.h>
#include <process.h>
#include "decoder.h"

#define WM_PLAYOVER WM_USER+40

MPArgs *Args;
struct frame fr;
HANDLE ThreadQueueEvent = NULL;
HANDLE SeekEvent, SeekEvent2, FinishEvent;
int NewSeekPosition;

int BuffersType;
int InputBuffersType;
int ForceMono;
int Downsampling;

void HardAudioReset(MPPlayer *ptr)
{
    FinishBuffer();
    try {
        Done_MPAudio();
    }
    catch(...) {}
    try {
        Init_MPAudio();
    }
    catch(...) {
        throw 1;            
        return;
    }
}

void SynthesisFilterInit()
{
    typedef int (*func)(real *,int,unsigned char * , int *);
    typedef int (*func_mono)(real *,unsigned char * , int *);
    
    static func funcs[3] = 
    {
        synth_1to1,
        synth_2to1,
        synth_4to1
    };
    
    static func_mono funcs_mono[2][3] = 
    {
        {	synth_1to1_mono2stereo ,
			synth_2to1_mono2stereo ,
			synth_4to1_mono2stereo ,
        } ,
        {   synth_1to1_mono ,
			synth_2to1_mono ,
			synth_4to1_mono
		}
    };
         
    fr.synth = (synthProc) funcs[Args->DownSample];
    fr.synth_mono = funcs_mono[Args->ForceMono][Args->DownSample];
    
    fr.block_size = 128 >> (Args->ForceMono + Args->DownSample);
    }

unsigned long __stdcall PlayThread(void *ptr) 
{
    MMRESULT mmRes;
	MPPlayer *Player = (MPPlayer *)ptr;
	  
	SynthesisFilterInit();
    try {
        Init_MPEGDecoder(Args->DownSample);
    }
    catch(...){
		Player->FPlayerMode = pmClosed;
        SetEvent(ThreadQueueEvent);
        SetEvent(FinishEvent);
        ExitThread(0);
        return 0;
    }

    try 
    {
        Init_MPAudio();
    }
    catch(...)
    {
		Done_MPEGDecoder();
		Player->FPlayerMode = pmClosed;
        SetEvent(ThreadQueueEvent);
        SetEvent(FinishEvent);
        ExitThread(0);
        return 0;
    }

	if(!InitInputBuffer())
	{
		Done_MPEGDecoder();
		Done_MPAudio();
		Player->FPlayerMode = pmClosed;
        SetEvent(ThreadQueueEvent);
        SetEvent(FinishEvent);
        ExitThread(0);
        return 0;
	}
	
	Player->FPlayerMode = pmPlaying;
    SetEvent(ThreadQueueEvent);

	long NewSF;
	BOOL Res;

	for(;;)
	{
		try 
		{
            if (WaitForSingleObject(SeekEvent, 0) == WAIT_OBJECT_0) 
            {
                if (Args->OutputMode == omMMSystem) 
                {
					AudioFlush();
                    mmRes = waveOutReset(WaveDevice);
                    ProcessAudioMessages();
                    BufferCount = 0;
                }
                if (Seek_Stream(NewSeekPosition))
                    NewSeekPosition = -1;
                else
                    NewSeekPosition = 0;
                SetEvent(SeekEvent2);
				if(NewSeekPosition == 0)
					break;
            }
            ProcessAudioMessages();

			FillInputBuffer();

			Res=GetNextFrame();

            if(Player->UserBreak || !Res) 
            {
                break;
            }
            
			Args->CurrentPos++;

			NewSF=freqs[fr.sampling_frequency]/(Args->DownSample==2?4:Args->DownSample==1?2:1);
			
			if(Args->SampleRate!=NewSF)
			{
				Args->SampleRate=NewSF;
				HardAudioReset(Player);
			}

            if(fr.error_protection) GetBits(16);          
			if(fr.lay==3) do_layer3();
			else if(fr.lay==2) do_layer2();
			else if(fr.lay==1) do_layer1();
            if (Player->PauseFlag) 
            {
                if (Args->OutputMode == omMMSystem) Pause_Audio();
                Player->ThreadSync = 1;  
                while(Player->PauseFlag)
                {
                    Sleep(77);
                }
                if (Args->OutputMode == omMMSystem && !Player->UserBreak) Resume_Audio();
            } 
		} catch (...) 
		{
		}
	}
	FreeInputBuffer();
    if(!Player->UserBreak)
    {
		try 
		{ 
			AudioFlush();
			FinishBuffer(); 
		} 
		catch (...) 
		{
		}
		if (Decoder_Initialized) 
		{
			Done_MPEGDecoder();
			Decoder_Initialized = false;        
		}
		Done_MPAudio();
		Player->FPlayerMode = pmClosed;
		Player->UserBreak=0; 
		if (WaitForSingleObject(SeekEvent, 0) == WAIT_OBJECT_0) 
		{
			NewSeekPosition = 0;
			SetEvent(SeekEvent2);
		}
		SetEvent(FinishEvent);
		if(Args->wnd!=NULL)
			::PostMessage(Args->wnd,WM_PLAYOVER,0,0);
		ExitThread(0);
		return 0;
    }
    else
    {
        if (Args->OutputMode == omMMSystem)
        {
            AudioFlush();
            mmRes = waveOutReset(WaveDevice);
            ProcessAudioMessages();
            BufferCount = 0;
        }
        if (Decoder_Initialized) 
        {
            Done_MPEGDecoder();
            Decoder_Initialized = false;        
        }
        Done_MPAudio();
        Player->FPlayerMode = pmClosed;
		Player->UserBreak=0; 
		if (WaitForSingleObject(SeekEvent, 0) == WAIT_OBJECT_0) 
		{
			NewSeekPosition = 0;
			SetEvent(SeekEvent2);
		}
		SetEvent(FinishEvent);
		ExitThread(0);
		return 0;
    }
}

MPPlayer::~MPPlayer() {
    if (PlayerMode() > pmClosed) Close();
    if (Args) delete Args;
    CloseHandle(FinishEvent);
    CloseHandle(SeekEvent);
    CloseHandle(SeekEvent2);
}

MPPlayer::MPPlayer() {
    FinishEvent = CreateEvent(NULL, false, false, NULL);
    SeekEvent = CreateEvent(NULL, false, false, NULL);
    SeekEvent2 = CreateEvent(NULL, false, false, NULL);
    Args = new MPArgs();
    if (!Args) throw 1;
    Args->InputMode=imFile;
    Args->OutputMode=omMMSystem;  
    Args->Player = this;
    FPlayerMode = pmClosed;
    UserBreak = false;
    fr.single = -1;     
    
    for(int i=0; i<32; i++) {
        equalizer[i]=1.0;
    }
};

void MPPlayer::SetInName (const char *FileName){
	if((FileName[0]=='h'||FileName[0]=='H')&&
	   (FileName[1]=='t'||FileName[1]=='T')&&
	   (FileName[2]=='t'||FileName[2]=='T')&&
	   (FileName[3]=='p'||FileName[3]=='P')&&
	   FileName[4]==':'&&
	   FileName[5]=='/'&&
	   FileName[6]=='/'||
	   (FileName[0]=='f'||FileName[0]=='F')&&
	   (FileName[1]=='t'||FileName[1]=='T')&&
	   (FileName[2]=='p'||FileName[2]=='P')&&
	   FileName[3]==':'&&
	   FileName[4]=='/'&&
	   FileName[5]=='/')
	{
		Args->InputMode = imHTTP;
	}
	else
	{
		Args->InputMode = imFile;
	}

    if (Args->InName) free(Args->InName);
    Args->InName = (char *) xstrdup((LPTSTR) FileName);
}

void MPPlayer::SetOutName(const char *FileName){
    if (Args->OutName) free(Args->OutName);
    Args->OutName = (char *) xstrdup((LPTSTR) FileName);
}

void MPPlayer::SetOutMode(esOutputMode OutMode){
    Args->OutputMode = OutMode;
}

bool MPPlayer::InitStream(){
	PauseFlag = 0;
    try {
        Init_Stream();            
    } catch (...) {
        return false;
    }
    
    try 
    {
        memset(&fr, 0, sizeof(fr));
        fr.single = -1;
        if (!ReadFrame(&fr)) 
        {
            Done_Stream();
            return false;
        }
        Reset_Stream();
    } 
    catch ( ... ) 
    {
		Done_Stream();
        return false;
    }
	
    FPlayerMode = pmReady;
    return true;
}

void MPPlayer::Play() {
	switch(BuffersType)
	{
		case 0:
			Args->Buffers=1;
			Args->BufferSize=16384;
			break;
		case 1:
			Args->Buffers=4;
			Args->BufferSize=16384;
			break;
		case 2:
			Args->Buffers=8;
			Args->BufferSize=16384;
			break;
		case 3:
			Args->Buffers=8;
			Args->BufferSize=32768;
			break;
		case 4:
			Args->Buffers=10;
			Args->BufferSize=32768;
			break;
	}

	switch(InputBuffersType)
	{
		case 0:
			Args->InpBufFrames=10;
			break;
		case 1:
			Args->InpBufFrames=50;
			break;
		case 2:
			Args->InpBufFrames=100;
			break;
		case 3:
			Args->InpBufFrames=150;
			break;
		case 4:
			Args->InpBufFrames=200;
			break;
	}

	Args->ForceMono=ForceMono;
	Args->DownSample=Downsampling;

	Args->SampleRate = freqs[fr.sampling_frequency];

	Args->Channels=2;

    if(Args->ForceMono) {
        if(fr.single<0) fr.single=3;
        Args->Channels=1;
    }
	
    if(Args->DownSample==1) 
    {
        Args->SampleRate = Args->SampleRate/2;
    }
    if(Args->DownSample==2) 
    {
        Args->SampleRate = Args->SampleRate/4;
    }

    ThreadQueueEvent = CreateEvent(NULL, false, false, NULL);
    if (!ThreadQueueEvent) 
    {
        return;
    }
    ResetEvent(FinishEvent);
    if (PlayThreadHan)
    {
        CloseHandle(PlayThreadHan);
        PlayThreadHan = NULL;
    }
    PlayThreadHan = CreateThread(NULL, 0, PlayThread, this, 0, &PlayerThreadID);
    if (!PlayThreadHan) 
    {
        return;           
    }
	SetThreadPriority(PlayThreadHan, Args->PlayPriority);

    WaitForSingleObject(ThreadQueueEvent, INFINITE);
    CloseHandle(ThreadQueueEvent);
	if(PlayerMode() != pmPlaying)
		Done_Stream();
}

void MPPlayer::Pause(){
    PauseFlag = true;
    FPlayerMode = pmPaused;
}

void MPPlayer::Resume(){
    PauseFlag = false;   
    FPlayerMode = pmPlaying;
}

void MPPlayer::Close(){
	if ((PlayerMode() == pmPlaying) || (PlayerMode() == pmPaused))
	{
        UserBreak = 1;
		PauseFlag = false;
        WaitForSingleObject(FinishEvent, INFINITE);
        if (PlayThreadHan!=NULL)
        {
            CloseHandle(PlayThreadHan);
            PlayThreadHan = NULL;
        }
	}
    if (StreamInit) {
        Done_Stream();
        StreamInit = false;
    }    
    FPlayerMode = pmClosed;
}

unsigned long MPPlayer::GetSize(){
    return Totalms();
}

unsigned long MPPlayer::GetPos()
{
   return Currentms();
}

bool MPPlayer::CanSetPos() 
{
	if (PlayerMode() == pmPlaying) 
        return Args->Seekable;
    else 
        return false;
}

void MPPlayer::SetPos(int Percent){
	if(Percent<0) Percent=0;
    NewSeekPosition = Percent;
    SetEvent(SeekEvent);
    WaitForSingleObject(SeekEvent2, INFINITE);
}

void MPPlayer::SetPriority(long Priority) 
{
	Args->PlayPriority = Priority;
	if(PlayThreadHan)
		SetThreadPriority(PlayThreadHan, Priority);
}

esPlayerMode MPPlayer::PlayerMode() {
    return FPlayerMode;
}

void MPPlayer::SetWindow(HWND Wnd) {
    Args->wnd = Wnd;
}
