#include <windows.h>
#include "Volume.h"

#define	BAD_DWORD	(DWORD)0

extern int MDevice;

CVolume::CVolume(int type)
	:	m_bAvailable(false),

		m_uMixerID(0L),
		m_dwMixerHandle(0L),

		m_dwMinimalVolume(BAD_DWORD),
		m_dwMaximalVolume(BAD_DWORD)
{
	if (mixerGetNumDevs()<1)
		return;
	MMRESULT mmResult;
	mmResult=mixerOpen((LPHMIXER)&m_dwMixerHandle, MDevice, 0, 0L, CALLBACK_NULL);
	if (mmResult != MMSYSERR_NOERROR)
		return;
	MIXERLINE MixerLine;
	memset( &MixerLine, 0, sizeof(MIXERLINE) );
	MixerLine.cbStruct = sizeof(MIXERLINE);
	MixerLine.dwComponentType = (type==0?MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT:MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC);
	mmResult = mixerGetLineInfo( (HMIXEROBJ)m_dwMixerHandle, &MixerLine, MIXER_GETLINEINFOF_COMPONENTTYPE );
	if ( mmResult != MMSYSERR_NOERROR )
	{
		return;
	}

	MIXERCONTROL Control;
	memset( &Control, 0, sizeof(MIXERCONTROL) );
	Control.cbStruct = sizeof(MIXERCONTROL);

	MIXERLINECONTROLS LineControls;
	memset( &LineControls, 0, sizeof(MIXERLINECONTROLS) );
	LineControls.cbStruct = sizeof(MIXERLINECONTROLS);

	LineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	LineControls.dwLineID = MixerLine.dwLineID;
	LineControls.cControls = 1;
	LineControls.cbmxctrl = sizeof(MIXERCONTROL);
	LineControls.pamxctrl = &Control;
	mmResult = mixerGetLineControls( (HMIXEROBJ)m_dwMixerHandle, &LineControls, MIXER_GETLINECONTROLSF_ONEBYTYPE );
	if ( mmResult == MMSYSERR_NOERROR )
	{
		if ( !(Control.fdwControl & MIXERCONTROL_CONTROLF_DISABLED) )
		{
			m_bAvailable = true;
		}
	}

	m_nChannelCount = MixerLine.cChannels;
	m_dwLineID = LineControls.dwLineID;
	m_dwVolumeControlID = Control.dwControlID;
	m_dwMinimalVolume = Control.Bounds.dwMinimum;
	m_dwMaximalVolume = Control.Bounds.dwMaximum;
	m_dwVolumeStep = Control.Metrics.cSteps;
}

CVolume::~CVolume()
{
	if(m_dwMixerHandle)
		mixerClose( (HMIXER)m_dwMixerHandle );
}

void CVolume::SetCurrentVolume( DWORD dwValue )
{
	if ( !m_bAvailable)
		return;
	dwValue=(m_dwMaximalVolume-m_dwMinimalVolume)*dwValue/100 + m_dwMinimalVolume;
	if(dwValue==0)
	{
		dwValue=(m_dwMaximalVolume-m_dwMinimalVolume)/1000;
	}
	MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
	if(!aDetails) return;
	double* d = (double*)malloc(m_nChannelCount*sizeof(double));
	if (!d) return;
	MIXERCONTROLDETAILS ControlDetails;
	memset( &ControlDetails, 0, sizeof(MIXERCONTROLDETAILS) );
	ControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
	ControlDetails.dwControlID = m_dwVolumeControlID;
	ControlDetails.cChannels = m_nChannelCount;
	ControlDetails.cMultipleItems = 0;
	ControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	ControlDetails.paDetails = &aDetails[0];
	MMRESULT mmResult;
	mmResult= mixerGetControlDetails( (HMIXEROBJ)m_dwMixerHandle, &ControlDetails, MIXER_GETCONTROLDETAILSF_VALUE );
	DWORD max=m_dwMinimalVolume;
	int i;
	for ( i = 0; i < m_nChannelCount; i++ )
	{
		if(aDetails[i].dwValue>max) max=aDetails[i].dwValue;
	}
	for ( i = 0; i < m_nChannelCount; i++ )
	{
		if(max>0)
			d[i]=(double)aDetails[i].dwValue/(double)max;
		else
			d[i]=1;
	}
	for ( i = 0; i < m_nChannelCount; i++ )
	{
		aDetails[i].dwValue = (int)((double)dwValue*d[i]);
	}
	free(d);

	mmResult = mixerSetControlDetails( (HMIXEROBJ)m_dwMixerHandle, &ControlDetails, MIXER_SETCONTROLDETAILSF_VALUE );
	free( aDetails );
}

DWORD CVolume::GetCurrentVolume()
{
	if ( !m_bAvailable )
		return BAD_DWORD;

	MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
	MIXERCONTROLDETAILS ControlDetails;
	memset( &ControlDetails, 0, sizeof(MIXERCONTROLDETAILS) );
	ControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
	ControlDetails.dwControlID = m_dwVolumeControlID;
	ControlDetails.cChannels = m_nChannelCount;
	ControlDetails.cMultipleItems = 0;
	ControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	ControlDetails.paDetails = &aDetails[0];
	MMRESULT mmResult;
	mmResult= mixerGetControlDetails( (HMIXEROBJ)m_dwMixerHandle, &ControlDetails, MIXER_GETCONTROLDETAILSF_VALUE );
	DWORD max=m_dwMinimalVolume;
	int i;
	for ( i = 0; i < m_nChannelCount; i++ )
	{
		if(aDetails[i].dwValue>max)
			max=aDetails[i].dwValue;
	}
	free( aDetails );
	max=100*(max-m_dwMinimalVolume)/(m_dwMaximalVolume-m_dwMinimalVolume);
	return max;
}
